﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.Text;
using Microsoft.Azure.Management.DataFactories.Models;
using Microsoft.Azure.Management.DataFactories.Runtime;
using Microsoft.WindowsAzure.Storage;
using Microsoft.WindowsAzure.Storage.Blob;
using TrialToTaskActivity.TaskService;

namespace TrialToTaskActivity
{
    public class CustomActivity : IDotNetActivity
    {
        public IDictionary<string, string> Execute(IEnumerable<LinkedService> linkedServices, IEnumerable<Dataset> datasets, Activity activity, IActivityLogger logger)
        {
            var inputDataset = datasets
                .Single(dataset => dataset.Name == activity.Inputs.Single().Name);

            var inputLinkedService = linkedServices
                .First(linkedService => linkedService.Name == inputDataset.Properties.LinkedServiceName)
                .Properties.TypeProperties as AzureStorageLinkedService;

            string connectionString = inputLinkedService.ConnectionString;
            string folderPath = (inputDataset.Properties.TypeProperties as AzureBlobDataset).FolderPath;
            string fileName = (inputDataset.Properties.TypeProperties as AzureBlobDataset).FileName;

            logger.Write($"ConnectionString = {connectionString}");
            logger.Write($"FolderPath = {folderPath}");
            logger.Write($"FileName = {fileName}");

            try
            {
                SubmitTasks(connectionString, folderPath, fileName);
                logger.Write("Done");
            }
            catch (Exception e)
            {
                logger.Write($"Exception: {e.Message}");
            }

            return new Dictionary<string, string>();
        }

        public void SubmitTasks(string connectionString, string folderPath, string fileName)
        {
            CloudStorageAccount inputStorageAccount = CloudStorageAccount.Parse(connectionString);
            CloudBlobClient inputClient = inputStorageAccount.CreateCloudBlobClient();
            BlobContinuationToken continuationToken = null;

            do
            {
                BlobResultSegment blobList = inputClient.ListBlobsSegmented(folderPath,
                                 true,
                                 BlobListingDetails.Metadata,
                                 null,
                                 continuationToken,
                                 null,
                                 null);

                foreach (var listBlobItem in blobList.Results.OfType<CloudBlockBlob>())
                {
                    var lastSlashIndex = listBlobItem.Name.LastIndexOf("/");
                    var blobFileName = listBlobItem.Name.Substring(lastSlashIndex + 1);
                    if (blobFileName == fileName)
                    {
                        var lines = listBlobItem.DownloadText(Encoding.ASCII)
                            .Split(new string[] { "\n", "\r\n" }, StringSplitOptions.RemoveEmptyEntries)
                            .Skip(1);

                        foreach (var line in lines)
                        {
                            var parts = line.Split(',');
                            var username = parts[1];
                            var start = parts[0];
                            var conversion = parts[12];
                            using (var client = new TaskServiceClient(new BasicHttpBinding(), new EndpointAddress("http://sdnadfcrm.cloudapp.net/TaskService.svc")))
                            {
                                client.AddTask($"Call back user {username} who started a trial at {start}. Probable conversion: {conversion}");
                            }
                        }
                    }
                }
            } while (continuationToken != null);
        }
    }
}
